@@ -32,8 +32,8 @@ This module aims to implement the W3C WebDriver specification which is the
 standardised equivalent to the Selenium WebDriver API., which in turn aims
 at remotely controlling web browsers such as Firefox or Chromium.
 
-At the time of this writing, it was so brand new that I ciould only get
-C<geckodriver> (For Firefox) to work, but that is expected to be fioxed
+At the time of this writing, it was so brand new that I could only get
+C<geckodriver> (For Firefox) to work, but that is expected to be fixed
 very soon indeed.
 
 To make most of this module, or, in fact, to make any reasonable use of
@@ -57,9 +57,11 @@ use Carp ();
 use AnyEvent ();
 use AnyEvent::HTTP ();
 
-our $VERSION = 0.5;
+our $VERSION = 0.9;
 
 our $WEB_ELEMENT_IDENTIFIER = "element-6066-11e4-a52e-4f735466cecf";
+our $WEB_WINDOW_IDENTIFIER  =  "window-fcc6-11e5-b4f8-330a88ab9d7f";
+our $WEB_FRAME_IDENTIFIER   =   "frame-075b-4da1-b6ba-e579c2d3230a";
 
 my $json = eval { require JSON::XS; JSON::XS:: } || do { require JSON::PP; JSON::PP:: };
 $json = $json->new->utf8;
@@ -72,9 +74,10 @@ sub req_ {
 
    AnyEvent::HTTP::http_request $method => "$self->{_ep}$ep",
       body => $body,
+      $self->{persistent} ? (persistent => 1) : (),
+      $self->{proxy} eq "default" ? () : (proxy => $self->{proxy}),
       timeout => $self->{timeout},
       headers => { "content-type" => "application/json; charset=utf-8", "cache-control" => "no-cache" },
-      ($self->{proxy} eq "default" ? () : (proxy => $self->{proxy})),
       sub {
          my ($res, $hdr) = @_;
 
@@ -128,7 +131,7 @@ sub AUTOLOAD {
 
          if (exists $res->{error}) {
             $msg = "AyEvent::WebDriver: $res->{error}: $res->{message}";
-            $msg .= "\n$res->{stacktrace}" if length $res->{stacktrace};
+            $msg .= "\n$res->{stacktrace}caught at" if length $res->{stacktrace};
          } else {
             $msg = "AnyEvent::WebDriver: http status $status (wrong endpoint?), caught";
          }
@@ -181,6 +184,16 @@ likely drastically reduce). This timeout is reset on any activity, so it
 is not an overall request timeout. Also, individual requests might extend
 this timeout if they are known to take longer.
 
+=item persistent => C<1> | C<undef>
+
+If true (the default) then persistent connections will be used for all
+requests, which assumes you have a reasonably stable connection (such as
+to C<localhost> :) and that the WebDriver has a persistent timeout much
+higher than what L<AnyEvent::HTTP> uses.
+
+You can force connections to be closed for non-idempotent requests by
+setting this to C<undef>.
+
 =back
 
 =cut
@@ -191,6 +204,7 @@ sub new {
    bless {
       endpoint   => "http://localhost:4444",
       proxy      => undef,
+      persistent => 1,
       autodelete => 1,
       timeout    => 300,
       %kv,
@@ -224,8 +238,8 @@ itself.
 
 The main use of this function is in conjunction with disabled
 C<autodelete>, to save a session to e.g., and restore it later. It could
-presumably used for other applications, suhc as using the same sssion from
-multiple processes and so on.
+presumably used for other applications, such as using the same session
+from multiple processes and so on.
 
 =item $wd->load_session ($sessionstring)
 
@@ -304,7 +318,7 @@ successfully, and only one session can be created per WebDriver object.
 On success, C<< $wd->{sid} >> is set to the session ID, and C<<
 $wd->{capabilities} >> is set to the returned capabilities.
 
-Simple example of creatring a WebDriver object and a new session:
+Simple example of creating a WebDriver object and a new session:
 
    my $wd = new AnyEvent::Selenium endpoint => "http://localhost:4545";
    $wd->new_session ({});
@@ -326,6 +340,7 @@ Real-world example with capability negotiation:
                   args => ["-devtools"],
                   prefs => {
                      "dom.webnotifications.enabled" => \0,
+                     "dom.push.enabled" => \0,
                      "dom.disable_beforeunload" => \1,
                      "browser.link.open_newwindow" => 3,
                      "browser.link.open_newwindow.restrictions" => 0,
@@ -538,14 +553,14 @@ sub switch_to_parent_frame_ {
 
 =item $rect = $wd->get_window_rect
 
-Return the current window rect, e.g.:
+Return the current window rect(angle), e.g.:
 
    $rect = $wd->get_window_rect
    => { height => 1040, width => 540, x => 0, y => 0 }
 
 =item $wd->set_window_rect ($rect)
 
-Sets the window rect.
+Sets the window rect(angle).
 
 =cut
 
@@ -615,6 +630,8 @@ sub _using($) {
 Finds the first element specified by the given selector and returns its
 element object. Raises an error when no element was found.
 
+Examples showing all standard locator strategies:
+
    $element = $wd->find_element ("css selector" => "body a");
    $element = $wd->find_element ("link text" => "Click Here For Porn");
    $element = $wd->find_element ("partial link text" => "orn");
@@ -622,6 +639,13 @@ element object. Raises an error when no element was found.
    $element = $wd->find_element ("xpath" => '//input[@type="text"]');
    => e.g. { "element-6066-11e4-a52e-4f735466cecf" => "decddca8-5986-4e1d-8c93-efe952505a5f" }
 
+Same examples using aliases provided by this module:
+
+   $element = $wd->find_element (css => "body a");
+   $element = $wd->find_element (link => "Click Here For Porn");
+   $element = $wd->find_element (substr => "orn");
+   $element = $wd->find_element (tag => "input");
+
 =item $elements = $wd->find_elements ($locator_strategy, $selector)
 
 As above, but returns an arrayref of all found element objects.
@@ -931,12 +955,12 @@ example:
    my $input = $wd->find_element ("css selector", 'input[type="text"]');
    $wd->actions
       ->move ($input, 40, 5, "touch1")
-      ->click;
-      ->key ("a");
-      ->key ("b");
-      ->pause (2000);
+      ->click
+      ->key ("a")
+      ->key ("b")
+      ->pause (2000)
       ->key ("\x{E007}")
-      ->pause (5000);
+      ->pause (5000)
       ->perform;
 
 =item $wd->release_actions
@@ -1070,12 +1094,14 @@ your own sources and use them when adding events. The above example could
 be more verbosely written like this:
 
    $wd->actions
+      ->source ("mouse", "pointer", pointerType => "mouse")
+      ->source ("kbd", "key")
       ->click (1, 100, "mouse")
-      ->type ("some text")
-      ->key ("{Enter}")
+      ->type ("some text", "kbd")
+      ->key ("{Enter}", "kbd")
       ->perform;
 
-When you specify the event source expliticly it will switch the current
+When you specify the event source explicitly it will switch the current
 "focus" for this class of device (all keyboards are in one class, all
 pointer-like devices such as mice/fingers/pens are in one class), so you
 don't have to specify the source for subsequent actions.
@@ -1109,7 +1135,7 @@ sub new {
 
 =item $al = $al->source ($id, $type, key => value...)
 
-The first time you call this with a givne ID, this defines the event
+The first time you call this with a given ID, this defines the event
 source using the extra parameters. Subsequent calls merely switch the
 current source for its event class.
 
@@ -1120,10 +1146,14 @@ Example: define a new touch device called C<fatfinger>.
 
    $al->source (fatfinger => "pointer", pointerType => "touch");
 
-Example: switchdefine a new touch device called C<fatfinger>.
+Example: define a new touch device called C<fatfinger>.
 
    $al->source (fatfinger => "pointer", pointerType => "touch");
 
+Example: switch default keyboard source to C<kbd1>, assuming it is of C<key> class.
+
+   $al->source ("kbd1");
+
 =cut
 
 sub _default_source($) {
@@ -1443,7 +1473,7 @@ sub type {
 
 =item $al->perform ($wd)
 
-Finaluses and compiles the list, if not done yet, and calls C<<
+Finalises and compiles the list, if not done yet, and calls C<<
 $wd->perform >> with it.
 
 If C<$wd> is undef, and the action list was created using the C<<
